% This is one solution to the Day 5 HW: Image processing. Keep in mind this
% solution has been highly customized to these particular images and might
% not work in other situations. In particular, there are several
% parameters in the labelCells() function that were chosen only after
% trials and error with these images.

clear
close all

%% Count cells
% Count cells in the red and blue images. Make sue you look at labelCells
% to understand what this is doing. If you run this function with a second
% input of 1, e.g. labelCells('RFP_Time2.TIF',1), then figures will be 
% produced showing each processing step

[redLabel,nRed,RFP] = labelCells('RFP_Time2.TIF');
[blueLabel,nBlue,BFP] = labelCells('BFP_Time2.TIF');

% Load YFP image for data analysis and convert to double
YFP = double(imread('YFP_Time2.TIF'));

%% Visualize label results

% Create overlay of original image and label to show if result is valid.
overlayRFP = labeloverlay(RFP,redLabel);
figure, imshow(overlayRFP)
title(['RFP overlay with ' num2str(nRed) ' cells'])

overlayBFP = labeloverlay(BFP,blueLabel);
figure, imshow(overlayBFP)
title(['BFP overlay with ' num2str(nBlue) ' cells'])


% Combine labels together and overlay on YFP image
combinedLabel = (redLabel>0) + 2*(blueLabel>0);
colors = [1,0,0; 0,0,1];
overlayYFP = labeloverlay(rescale(YFP),combinedLabel,'Colormap',colors);

% Plot the overlay
figure, imshow(overlayYFP)
title('YFP image with red and blue cell overlay')

%% Get mean YFP values for each cell

% Get mean YFP values of RFP cells
redYFP = zeros(nRed,1);
for iRed = 1:nRed
    
    isCell = redLabel == iRed;
    redYFP(iRed) = mean(YFP(isCell));
end

% Repeat for blue
blueYFP = zeros(nBlue,1);
for iBlue = 1:nBlue
    
    isCell = blueLabel == iBlue;
    blueYFP(iBlue) = mean(YFP(isCell));
end


%% Compare with histograms

% Maker sure both histograms use same bins
binCenters = 105:10:205;

redCounts = hist(redYFP,binCenters);
blueCounts = hist(blueYFP,binCenters);

figure, hold on
hist(redYFP,binCenters);
hist(blueYFP,binCenters);
xlabel('Mean YFP value')
ylabel('Frequency')
legend('Red cells','Blue cells')

% Find histogram objects so we can change properties like color. This is a
% 2x1 array. histObjects(1) is the handle to blueYFP, since it was plotted
% most recently.
histObjects = findobj(gca,'Type','patch');

% Set Color and FaceAlpha to make it easier to see
set(histObjects(1),'FaceColor','b','FaceAlpha',0.5)
set(histObjects(2),'FaceColor','r','FaceAlpha',0.5)


%% Compare values with permutation test 

% Define our test statistic (difference of means). Our null hypotheses is
% that the difference is zero
meanDiff = mean(blueYFP) - mean(redYFP);
allData = [redYFP;blueYFP];

% Do permutations
nPerm = 1e5;
permDiff = zeros(nPerm,1);
for iPerm = 1:nPerm
    mixedData = allData(randperm(length(allData)));
    permDiff(iPerm) = mean(mixedData(1:nBlue)) - mean(mixedData(nBlue+1:end));    
end

% Calculate two-tailed pvalue
pValue = sum(abs(permDiff)>=abs(meanDiff)) / nPerm

% Make histogram of differences
figure;
hist(permDiff,100)
xlabel('Sampled mean differences')
ylabel('Frequency')
title('Permutation test')

% Get ylimits so we don't need to guess the length of the lines
yLimits = get(gca,'YLim');

%Add lines for +/- actual
line(abs(meanDiff)+[0 0],yLimits,'Color','r','LineWidth',2)
line(-abs(meanDiff)+[0 0],yLimits,'Color','r','LineWidth',2)
